{================================================}
{============= Score handling and display ==============}
{================================================}

{ Example file for Ingemars Sprite Animation Toolkit. }
{  Ingemar Ragnemalm 1992 }
{ See doc files for legal terms for using this code. }

{ This file manages the display and update of the game scores for HeartQuest.}
{ It holds routines for updating high score list, including asking for the name of}
{ the player, high score window etc. When making a new game, you will probably}
{ need to rewrite this unit a lot. }

unit scores;

interface
	uses
		transskel, SAT, GameGlobals, SoundConst;

	var
		score: longint;

	procedure DoHighMenu (item: integer);
	procedure initScores;					{ Loads the high score list and the high score window. }
	procedure zeroscore; 					{ Call this on New Game! }
	procedure addscore (amount: longint);	{ Call this when the player gets points, or with addscore(0) just to redisplay. }
	procedure addscoreS (amount: longint);	{ Call this to redisplay when the animation isn't running. }
	procedure UpdateHigh;					{ Call this on game over! }

implementation

{ Highscore record }
	type
		hsRec = record
				HighScores: array[0..10] of longint;
				HighPlayer: array[0..10] of str255;
			end;
		hsPtr = ^hsRec;
		hsHnd = ^hsPtr;


	var
		hs, hsm: hsHnd; { m is for macho mode }
		hsh, hshm: Handle;

{ Bitmaps and rects for drawhex }
{    srcr, erasr: Rect;}
{    bm: BitMap;}
{    icon: Handle;}
{    bma: array[0..9] of BitMap;}
{    icons: array[0..9] of Handle;}

{Filter function for AskHigh, ok = 1 and cancel = 4}
	function Filter (theDialog: DialogPtr; var theEvent: EventRecord; var itemHit: integer): boolean;
		var
			theChar: Char;
			kind: integer;
			item: Handle;
			box: Rect;
	begin
		if theEvent.what = keyDown then
			begin
				theChar := Char(BitAnd(theEvent.message, charCodeMask));
				if BitAnd(theEvent.modifiers, cmdkey) <> 0 then
					if theChar = '.' then
						begin
							itemHit := 4;
{Highlight the cancel button}
							GetDItem(theDialog, 4, kind, item, box);
							HiliteControl(ControlHandle(item), 1);

							Filter := true;
							exit(Filter);
						end;
				if (theChar = char(13)) or (theChar = char(3)) then
					begin
						itemHit := 1;
{Highlight the OK button}
						GetDItem(theDialog, 1, kind, item, box);
						HiliteControl(ControlHandle(item), 1);

						Filter := true;
						exit(Filter);
					end;
			end;
		Filter := false;
	end;

{Put a frame around a dialog item. There are better ways to do this, though. The right way}
{is to draw the frame as response to an update event, not just when opening the dialog.}
	procedure FrameDItem (dLog: DialogPtr; iNum: integer);
		var
			iBox: Rect;
			iType: integer;
			iHandle: Handle;
			oldPenState: PenState;
			tmpp: GrafPtr;
	begin
		GetPort(tmpp);
		SetPort(dLog);
		GetPenState(oldPenState);
		GetDItem(dLog, iNum, iType, iHandle, iBox);
		InsetRect(iBox, -4, -4);
		PenSize(3, 3);
		FrameRoundRect(iBox, 16, 16);
		SetPenState(oldPenState);
		SetPort(tmpp);
	end;

{ Ask for players name (at highscore) }
	function AskHigh: str255;
		var
			dialog: DialogPtr;
			oldPort: GrafPtr;
			dRec: DialogRecord;
			itemHit: integer;
			itemHandle: Handle;
			itemType, item: integer;
			itemRect: Rect;
			str: str255;
			levelstr: str255;
	begin
		GetPort(oldPort);
		dialog := GetNewDialog(HighDlog, @dRec, WindowPtr(-1));
		ShowWindow(dialog);
		SelectWindow(dialog);
		SetPort(dialog);

		GetDItem(dialog, 3, itemType, itemHandle, itemRect);
		SetIText(itemHandle, features^^.player);
		SelIText(dialog, 3, 0, 32767);
		FrameDItem(dialog, 1);
		itemHit := -1;
		while (itemHit <> 1) and (itemHit <> 4) do { 1=ok, 4=cancel }
			ModalDialog(@Filter, itemHit);
		if itemHit = 4 then
			begin
				AskHigh := '';
			end;
		if itemHit = 1 then
			begin
				GetDItem(dialog, 3, itemType, itemHandle, itemRect);
				GetIText(itemHandle, str);
				features^^.player := str;
{NumToString(level, levelstr); { A fix to show levels }
				AskHigh := str; {StringOf(str, ' (', levelstr, ')');}
			end;
		CloseDialog(dialog);
		SetPort(oldPort);
	end;

{ 	High Score window handlers }

	procedure HighUpdate (resized: boolean);
		var
			s: str255;
			i: integer;
	begin
		EraseRect(theHigh^.portrect);
		TextSize(9);

		moveto(10, 20);
		DrawString('Normal high score list');
		moveto(150, 20);
		DrawString('Macho high score list');
		moveto(0, 22);
		lineto(500, 22);
		moveto(140, 0);
		lineto(140, 400);

		for i := 1 to 10 do
			begin
				if not LastMacho and (i = LastHigh) then
					begin
						TextFace([bold]);
						ForeColor(RedColor);
					end;
				moveto(10, i * 18 + 20);
				DrawString(hs^^.HighPlayer[i]);
				moveto(110, i * 18 + 20);
				NumToString(hs^^.HighScores[i], s);
				DrawString(s);

				TextFace([]);
				ForeColor(BlackColor);
				if LastMacho and (i = LastHigh) then
					begin
						TextFace([bold]);
						ForeColor(RedColor);
					end;
				moveto(150, i * 18 + 20);
				DrawString(hsm^^.HighPlayer[i]);
				moveto(250, i * 18 + 20);
				NumToString(hsm^^.HighScores[i], s);
				DrawString(s);

				TextFace([]);
				ForeColor(BlackColor);
			end;
		TextSize(12);
	end;

	procedure HighHalt;
	begin
		CloseWindow(theHigh);
	end;

	function InternalAddScore (amount: longint): Rect;
		var
			s: str255;
			r: Rect;
	begin
		score := score + amount;

		SetPort(backScreen); {gameWind);}
{SetRect(r, 290, 285, 500, 310);}
{SetRect(r, offSizeH - 47, 15, offSizeH - 4, 150);}
		SetRect(r, offSizeH - 49, 14, offSizeH - 2, 155);
		EraseRoundRect(r, 10, 10);
		FrameRoundRect(r, 10, 10);
{FrameRect(r);}
		NumToString(Score, s);
		MoveTo(offSizeH - 47, 30);
		DrawString('Score: ');
		MoveTo(offSizeH - 47, 50);
		DrawString(s);

		NumToString(bonus, s);
{MoveTo(300, 300);}
		MoveTo(offSizeH - 47, 80);
		DrawString('Bonus: ');
		MoveTo(offSizeH - 47, 100);
		DrawString(s);

		NumToString(Level, s);
{MoveTo(300, 300);}
		MoveTo(offSizeH - 47, 130);
		DrawString('Level: ');
		MoveTo(offSizeH - 47, 150);
		DrawString(s);
{InsetRect(r, -1, -1); {Include frame?}
		InternalAddScore := r;
	end;

	procedure addscore (amount: longint);
		var
			s: str255;
			r: Rect;
			tmpport: grafptr;
	begin
		GetPort(tmpPort);
		r := InternalAddScore(amount);
		SATBackChanged(r); {Nya sttet att dunka saker p skrmen!}
		SetPort(tmpPort);
	end;

	procedure addscoreS (amount: longint);
		var
			s: str255;
			r: Rect;
			tmpport: grafptr;
	begin
		GetPort(tmpPort);
		r := InternalAddScore(amount);
		CopyBits(backScreen^.portbits, gameWind^.portBits, r, r, srcCopy, nil);
		CopyBits(backScreen^.portbits, offScreen^.portBits, r, r, srcCopy, nil);
		SetPort(tmpPort);
	end;

	procedure DoHighMenu (item: integer);
		var
			p: procptr;
			i: integer;
	begin
		case item of
			showhs: 
				begin
					ShowWindow(theHigh);
					SelectWindow(theHigh);
				end;
			clearhs: 
				begin
					if QuestionStr('Are you sure you want to erase the high scores?') then
						begin
							for i := 1 to 10 do
								begin
									hs^^.HighScores[i] := 0;			{ skall lsas frn fil eller resurs }
									hs^^.HighPlayer[i] := 'Noone';
									hsm^^.HighScores[i] := 0;			{ skall lsas frn fil eller resurs }
									hsm^^.HighPlayer[i] := 'Noone';
								end;
							hs^^.HighScores[0] := 10000;			{ Lowscore }
							hsm^^.HighScores[0] := 10000;			{ Lowscore }
							ChangedResource(handle(hs));
							ChangedResource(handle(hsm));
							HideWindow(theHigh);
						end;
				end;
			otherwise
				;
		end;
	end;

	procedure WindKey (theChar: char; theMods: integer);
	begin
	end;

{ Call this on game over! }
	procedure updatehigh;
		var
			num: integer;
			name, s: str255;
	begin
		LastMacho := features^^.macho;

		if features^^.macho then
			begin
				if score > hsm^^.HighScores[10] then
					begin
						num := 10;
						name := AskHigh;
						if name = '' then { alt length(name) = 0 }
							exit(updatehigh);
						while (hsm^^.HighScores[num - 1] < score) and (num > 1) do
							begin
								hsm^^.HighScores[num] := hsm^^.HighScores[num - 1];
								hsm^^.HighPlayer[num] := hsm^^.HighPlayer[num - 1];
								num := num - 1;
							end;
						LastHigh := num; {Remember last high for the highscore display}
						hsm^^.HighScores[num] := score;
						NumToString(level, s);
						hsm^^.HighPlayer[num] := stringof(name, ' (', s, ')'); {AskHigh;}
{hsh := hs;}
						ChangedResource(handle(hsm));
						HideWindow(theHigh);
						ShowWindow(theHigh);
						SelectWindow(theHigh);
					end;
			end{ if macho }
		else if score > hs^^.HighScores[10] then
			begin
				num := 10;
				name := AskHigh;
				if name = '' then { alt length(name) = 0 }
					exit(updatehigh);
				while (hs^^.HighScores[num - 1] < score) and (num > 1) do
					begin
						hs^^.HighScores[num] := hs^^.HighScores[num - 1];
						hs^^.HighPlayer[num] := hs^^.HighPlayer[num - 1];
						num := num - 1;
					end;
				LastHigh := num; {Remember last high for the highscore display}
				hs^^.HighScores[num] := score;
				hs^^.HighPlayer[num] := name;
				ChangedResource(handle(hs));
				HideWindow(theHigh);
				ShowWindow(theHigh);
				SelectWindow(theHigh);
			end;
	end;

	procedure zeroscore;
	begin
		score := 0;
		LastHigh := -1;
	end;

	procedure initScores;
		var
			i: integer;
	begin
		LastHigh := -1; {no "last"}

		theHigh := GetNewWindow(theHighRes, nil, WindowPtr(-1));
		SetPort(theHigh);
		dummy := SkelWindow(theHigh, nil, @WindKey, @HighUpdate, nil, nil, @HighHalt, nil, false);

		hs := hsHnd(GetResource('Bst', 0));
		if hs = nil then
			begin
				hs := hsHnd(NewHandle(Sizeof(hsRec)));
				CheckNoMem(Ptr(hs));

				for i := 1 to 10 do
					begin
						hs^^.HighScores[i] := 0;
						hs^^.HighPlayer[i] := 'Noone';
					end;
				hs^^.HighScores[0] := 10000;			{ Lowscore }
				AddResource(handle(hs), 'Bst', 0, 'High scores');
			end;

		hsm := hsHnd(GetResource('Bst', 1));
		if hsm = nil then
			begin
				hsm := hsHnd(NewHandle(Sizeof(hsRec)));
				CheckNoMem(Ptr(hsm));
				for i := 1 to 10 do
					begin
						hsm^^.HighScores[i] := 0;			{ skall lsas frn fil eller resurs }
						hsm^^.HighPlayer[i] := 'Noone';
					end;
				hsm^^.HighScores[0] := 10000;			{ Lowscore }
				AddResource(handle(hsm), 'Bst', 1, 'High scores');
			end;

		score := 0;
	end;

end.